import json
import random
import time
import requests
from bs4 import BeautifulSoup
from loguru import logger

class Parser:
    @staticmethod
    def _write_to_json(data, path):
        with open(path, 'w+') as fp:
            json.dump(data, fp, indent=2)

    @staticmethod
    def _read_from_json(path):
        with open(path, 'r') as fp:
            return json.load(fp)

    @staticmethod
    def _soup_from_url(url):
        timesleep = random.uniform(0.35, 0.5)
        time.sleep(timesleep)
        r = requests.get(url)
        soup = BeautifulSoup(r.text, "html.parser")
        if r.status_code != 200:
            logger.info("hltv request failed with status code: " + str(r.status_code))
            raise
        return soup


import os
from datetime import datetime, timedelta
from .parser import Parser
from loguru import logger

BASE_URL = "https://www.hltv.org"


class HLTVMatch(Parser):
    def __init__(self, url):
        self.match_url = url
        self.match_id = self.match_url.split('/')[2]
        self.teams_name = None
        self.teams_link = None
        self.analytics_center_link = None
        self.match_maps = None
        self.datetime = None
        self.__get_match_attributes()

    def __get_match_attributes(self):
        soup = self._soup_from_url(BASE_URL + self.match_url)
        get_normed_link = lambda _: "/" + "/".join(_.split('/')[2:4])
        get_team_name = lambda _: _.split('/')[3]
        team1_link = soup.find("div", class_="team1-gradient").find('a')['href']
        team2_link = soup.find("div", class_="team2-gradient").find('a')['href']
        analytics_link = soup.find("a", class_="matchpage-analytics-center-container")['href']
        self.teams_link = (get_normed_link(team1_link), get_normed_link(team2_link))
        self.teams_name = (get_team_name(team1_link), get_team_name(team2_link))
        self.analytics_center_link = BASE_URL + analytics_link
        timestamp = int(soup.find("div", class_="timeAndEvent").find(class_="time")['data-unix'][0:-3])
        self.datetime = str(datetime.fromtimestamp(timestamp) - timedelta(hours=8))

    def parse_analytics_center(self, filename=None):
        soup = self._soup_from_url(self.analytics_center_link)
        output = []
        try:
            insights = self.parse_analytics_summary(soup)
            output.append(insights)
            if filename is not None:
                self._write_to_json(insights, f"{filename}_insights.json")
        except:
            logger.info("Failed __parse_analytics_summary(), parsed match has non-regular data\n")
        try:
            map_stats = self.parse_pick_ban_stats(soup)
            output.append(map_stats)
            if filename is not None:
                self._write_to_json(map_stats, f"{filename}_maps_stats.json")
        except:
            logger.info("Failed __parse_pick_ban_stats(), parsed match has non-regular data\n")
        try:
            players_stats = self.parse_head_to_head(soup)
            output.append(players_stats)
            if filename is not None:
                self._write_to_json(players_stats, f"{filename}_players_stats.json")
        except:
            logger.info("Failed __parse_head_to_head(), parsed match has non-regular data\n")
        return tuple(output)

    def parse_analytics_summary(self, soup=None, filename=None):
        if not soup:
            soup = Parser._soup_from_url(self.analytics_center_link)
        normed = lambda x: list(map(lambda _: _.text.lower().strip(), x))
        # normed(soup.find('div', attrs={"class", "analytics-info fadeUp"}).find_all("div"))
        insights_stats = []
        for i in range(2):
            container = soup.find(class_=f"analytics-insights-container team{i + 1}")
            plus_len = len(container.find_all(class_="fa fa-plus"))
            minus_len = len(container.find_all(class_="fa fa-minus"))
            plus = ['plus'] * plus_len if plus_len > 0 else ['none']
            minus = ['minus'] * minus_len if minus_len > 0 else ['none']
            indicator = plus + minus
            insights = normed(container.find_all(class_="analytics-insights-insight"))
            for j in range(len(insights)):
                row_data = {
                    "team": self.teams_name[i],
                    "indicator": indicator[j],
                    "insight": insights[j],
                    "match_id": self.match_id
                }
                insights_stats.append(row_data)
        if filename is not None:
            self._write_to_json(insights_stats, f"{filename}.json")
        return insights_stats

    def parse_pick_ban_stats(self, soup=None, filename=None):
        if not soup:
            soup = Parser._soup_from_url(self.analytics_center_link)
        map_stats = []
        rows = soup.find(class_="table-container gtSmartphone-only").find("tbody").find_all('tr')
        for ind, row in enumerate(rows):
            if ind % 2 == 0:
                map_name = row.find('div', class_='analytics-map-name').text.lower()
            team_name = self.teams_name[ind % 2]
            pick_percentage = row.find('td', class_='analytics-map-stats-pick-percentage').text
            ban_percentage = row.find('td', class_='analytics-map-stats-ban-percentage').text
            win_percentage = row.find('td', class_='analytics-map-stats-win-percentage').text
            games_played = row.find('td', class_='analytics-map-stats-played').text
            comment = row.find(class_="analytics-map-stats-comment").text
            row_data = {
                "analytics_map_name": map_name,
                "team": team_name,
                "analytics_map_stats_pick_percentage": pick_percentage,
                "analytics_map_stats_ban_percentage": ban_percentage,
                "analytics_map_stats_win_percentage": win_percentage,
                "analytics_map_stats_played": games_played,
                "analytics_map_stats_comment": comment,
                "match_id": self.match_id
            }
            map_stats.append(row_data)
        if filename is not None:
            self._write_to_json(map_stats, f"{filename}.json")
        return map_stats

    def parse_head_to_head(self, soup=None, filename=None):
        if not soup:
            soup = Parser._soup_from_url(self.analytics_center_link)
        players_stats = []
        team_container = soup.find_all(class_="table-container")  # [0] [1] teams=
        for i in range(2):
            nicknames = team_container[i].find_all(class_='player-nickname')
            three_month_stats = team_container[i].find_all(class_='table-3-months')[1:6]
            event_stats = team_container[i].find_all(class_='table-event')[1:6]
            for player in range(5):
                player_data = {
                    "player_team": self.teams_name[i],
                    "player_nickname": nicknames[player].text,
                    "table_3_months": three_month_stats[player].text,
                    "table_event": event_stats[player].text,
                    "match_id": self.match_id
                }
                players_stats.append(player_data)
        if filename is not None:
            self._write_to_json(players_stats, f"{filename}.json")
        return players_stats

    def is_parsed(self) -> bool:
        """Checks if match is already parsed and adds it to matches_config.json if not"""
        try:
            match_dict = Parser()._read_from_json("./configs/matches_config.json")
        except:
            logger.info("""Created ./configs/match_config.json for mapping""")
            os.makedirs(os.path.dirname("./configs/"), exist_ok=True)
            match_dict = {}
        if self.match_id not in match_dict:
            data = list(map(lambda _: _.replace("-", " "), self.teams_name)).copy()
            data.extend([self.datetime, self.match_id, self.match_url])
            match_dict[self.match_id] = data
            Parser()._write_to_json(match_dict, "./configs/matches_config.json")
            return False
        return True


import os
from datetime import datetime, timedelta
from cuid import cuid
from .parser import Parser
from loguru import logger
BASE_URL = "https://www.hltv.org"


class HLTVTeam(Parser):
    def __init__(self, url):
        self.team_url = url
        self.team_name = url.split("/")[2].lower()
        self.sliced_url = ("/stats/teams" + url).split("/")
        self.team_cuid = self.__get_cuid(self.team_name)
        self.match_id = None

    def __get_cuid(self, team_name):
        """
        Returns unique id for team name. If team name is not in team_config.json, creates new id and writes to file.
        """
        try:
            team_dict = self._read_from_json("./configs/team_config.json")
        except:
            logger.info("""Created ./configs/team_config.json for mapping {"team name" : "unique id"}""")
            os.makedirs(os.path.dirname("./configs/"), exist_ok=True)
            team_dict = {}
        if team_name not in team_dict:
            team_dict[team_name] = cuid()
            self._write_to_json(team_dict, "./configs/team_config.json")
        return team_dict[team_name]

    @staticmethod
    def __get_time_filter(months=0):
        """ Returns time filter for url"""
        if months == 0:
            return '?startDate=all'
        date = datetime.now()
        start_date = datetime.strftime(date - timedelta(days=int(30 * months)), '%Y-%m-%d')
        end_date = datetime.strftime(date, '%Y-%m-%d')
        return f'?startDate={start_date}&endDate={end_date}'

    def parse_all_stats(self, time_filter: int, filename: str = None):
        """ Parse all stats for team
        :param time_filter: 0 - all time, 1 - last month, 2 - last 2 months, 3 - last 3 months
        :param filename: str, filename for saving json
        :return: tuple of list of dicts with stats
        """
        output = []
        try:
            matches_stats = self.parse_matches(time_filter)
            output.append(matches_stats)
            if filename is not None:
                self._write_to_json(matches_stats, f"{filename}_matches_stats.json")
        except:
            logger.info("Failed parse_matches()")
        try:
            maps_stats = self.parse_maps(time_filter)
            output.append(maps_stats)
            if filename is not None:
                self._write_to_json(maps_stats, f"{filename}_maps_stats.json")
        except:
            logger.info("Failed parse_maps()")
        try:
            players_stats = self.parse_players(time_filter)
            output.append(players_stats)
            if filename is not None:
                self._write_to_json(players_stats, f"{filename}_players_stats.json")
        except:
            logger.info("Failed parse_players()")
        try:
            events_stats = self.parse_events(time_filter)
            output.append(events_stats)
            if filename is not None:
                self._write_to_json(events_stats, f"{filename}_events_stats.json")
        except:
            logger.info("Failed parse_events()")
        return tuple(output)

    def parse_matches(self, time_filter=3, filename=None):
        """
        Parse played matches stats for team;
        :param time_filter: int, 0 = all time, 1 - 1 month, 6 - 6 months, ...
        :param filename: str, filename to save json
        :return: list of dicts with matches stats
        """

        matches_url = "/".join(self.sliced_url[0:3]) + "/matches/" + "/".join(self.sliced_url[3:5])
        start_url = BASE_URL + matches_url + self.__get_time_filter(time_filter)
        soup = self._soup_from_url(start_url)

        matches_stats = []
        rows = soup.find_all('tr', attrs={"class": "group-1 first"}) + \
               soup.find_all('tr', attrs={"class": "group-2 first"}) + \
               soup.find_all('tr', attrs={"class": "group-1"}) + \
               soup.find_all('tr', attrs={"class": "group-2"})
        for row in rows:
            row_elements = row.find_all("td")

            row_date = row_elements[0].a
            row_event = row_elements[1].span
            row_opponent = row_elements[3].a
            row_map = row_elements[4].span
            row_result = row_elements[5].span
            row_flag = row_elements[6]
            norm = lambda _: _.text.lower()
            row_data = {
                "team": self.team_name,
                "date": norm(row_date),
                "event": norm(row_event),
                "opponent": norm(row_opponent).strip().replace(" ", "-"),
                "map": norm(row_map),
                "result": norm(row_result),
                "flag": row_flag.text,
                "team_cuid": self.team_cuid,
                "time_filter": str(time_filter)
            }
            if self.match_id is not None:
                row_data['match_cuid'] = self.match_id
            matches_stats.append(row_data)
        if filename is not None:
            self._write_to_json(matches_stats, f"{filename}.json")
        return matches_stats

    def parse_maps(self, time_filter=3, filename=None):
        """
        Parse played maps stats for team;
        :param time_filter: int, 0 - all time, 1 - 1 month, 6 - 6 months, ...
        :param filename: str, filename to save json
        :return: list of dicts with maps stats
        """

        maps_url = "/".join(self.sliced_url[0:3]) + "/maps/" + "/".join(self.sliced_url[3:5])
        start_url = BASE_URL + maps_url + self.__get_time_filter(time_filter)
        soup = self._soup_from_url(start_url)

        maps_stats = []
        map_names = soup.find_all("div", attrs={'class': 'map-pool-map-name'})
        map_stats = soup.find_all("div", attrs={'class': 'stats-rows standard-box'})

        for i in range(len(map_stats)):
            map_name = map_names[i]
            stats = list(map(lambda _: _.find_all('span')[1].text, map_stats[i].find_all('div')))
            map_data = {
                "team": self.team_name,
                "map": map_name.text.split("-")[0].rstrip().lower(),
                "wins_draws_losses": stats[0],
                "win_rate": stats[1],
                "total_rounds": stats[2],
                "round_win_perc_after_first_kill": stats[3],
                "round_win_perc_after_first_death": stats[4],
                "team_cuid": self.team_cuid,
                "time_filter": str(time_filter)
            }
            if self.match_id is not None:
                map_data['match_cuid'] = self.match_id
            maps_stats.append(map_data)
        if filename is not None:
            self._write_to_json(maps_stats, f"{filename}.json")
        return maps_stats

    def parse_players(self, time_filter=3, filename=None):
        """
        Parse players stats for team;
        :param time_filter: int, 0 - all times, 1 - 1 months, 6 - 6 months, ...
        :param filename: str, filename to save json
        :return: list of dicts with players stats
        """

        players_url = "/".join(self.sliced_url[0:3]) + "/players/" + "/".join(self.sliced_url[3:5])
        start_url = BASE_URL + players_url + self.__get_time_filter(time_filter)
        soup = self._soup_from_url(start_url)
        players_stats = []
        players = soup.find_all("tr")
        for i in range(1, 10):
            try:
                player = list(map(lambda _: _.text, players[i].find_all('td')))
                player_data = {
                    "team": self.team_name,
                    "player": player[0],
                    "maps": player[1],
                    "rounds": player[2],
                    "kd_diff": player[3],
                    "kd": player[4],
                    "rating": player[5],
                    "team_cuid": self.team_cuid,
                    "time_filter": str(time_filter)
                }
                if self.match_id is not None:
                    player_data['match_cuid'] = self.match_id
                players_stats.append(player_data)
            except:
                pass
        if filename is not None:
            self._write_to_json(players_stats, f"{filename}.json")
        return players_stats

    def parse_events(self, time_filter=3, filename=None):
        """Parse events stats for team
        :param time_filter: int, 0 - all times, 1 - 1 months, 6 - 6 months, ...
        :param filename: str, filename to save json
        :return: list of dicts with events stats
        """
        events_url = "/".join(self.sliced_url[0:3]) + "/events/" + "/".join(self.sliced_url[3:5])
        start_url = BASE_URL + events_url + self.__get_time_filter(time_filter)
        soup = self._soup_from_url(start_url)

        events_stats = []
        events = soup.find_all("tr")
        for i in range(1, len(events)):
            event = list(map(lambda _: _.text.lower(), events[i].find_all('td')))
            event_data = {
                "placement": event[0],
                "event": event[1],
                "team": self.team_name,
                "team_cuid": self.team_cuid,
                "time_filter": str(time_filter)
            }
            if self.match_id is not None:
                event_data['match_cuid'] = self.match_id
            events_stats.append(event_data)

        if filename is not None:
            self._write_to_json(events_stats, f"{filename}.json")
        return events_stats


from .parser import *
from .team import *
from .match import *
from .upcoming_matches import *
from loguru import logger
import sys
logger.add(sys.stdout, format="{YYYY-MM-DD HH:mm:ss} | {level} | {message}", level="INFO", backtrace=True, diagnose=True)






# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hltv_stats']

package_data = \
{'': ['*']}

install_requires = \
['bs4>=0.0.1,<0.0.2',
 'cuid>=0.3,<0.4',
 'loguru>=0.6.0,<0.7.0',
 'pytest>=7.2.1,<8.0.0',
 'requests>=2.28.2,<3.0.0']

setup_kwargs = {
    'name': 'hltv-stats',
    'version': '0.1.6',
    'description': 'Simple parser for HLTV.org team stats and matches (using requests and bs4)',
    'long_description': '<p>\n  <img alt="Version" src="https://img.shields.io/badge/version-0.1.6-blue.svg?cacheSeconds=2592000" />\n  <a href="#" target="_blank">\n    <img alt="License: MIT" src="https://img.shields.io/badge/License-MIT-yellow.svg" />\n  </a>\n</p>\n\n> Hi, I analyze hltv.org as a part of my pet project.\n> This parser can help you build a prematch analytics dataset with data from [Team stats]( https://www.hltv.org/stats/teams) and [Analytics](https://www.hltv.org/betting/analytics) pages.\n\n\n\n## Install\n\n```sh\npip install hltv-stats\n```\n\n## Usage\n#### ```HLTVMatch``` provides public methods for [Analytics](https://www.hltv.org/betting/analytics), use ```filename``` parameter to save data to a file.\n```sh\nfrom hltv_stats import HLTVMatch\nmatch_url = "/matches/2361342/natus-vincere-vs-outsiders-iem-katowice-2023"\nmatch = HLTVMatch(match_url)\n```\n```sh\nmatch.parse_analytics_summary(filename=None)\n```\n\n```sh\nResponse:\n\n[{\'team\': \'natus-vincere\',\n  \'indicator\': \'plus\',\n  \'insight\': \'natus vincere has better form ranking\',\n  \'match_id\': \'2361342\'},\n    ...\n]\n```\n```sh\nmatch.parse_head_to_head()\n```\n\n```sh\nResponse:\n\n[{\'player_team\': \'natus-vincere\',\n  \'player_nickname\': \'s1mple\',\n  \'table_3_months\': \'1.13\',\n  \'table_event\': \'1.17\',\n  \'match_id\': \'2361342\'},\n    ...\n]\n```\n```sh\nmatch.parse_pick_ban_stats()\n```\n\n```sh\nResponse:\n\n[{\'analytics_map_name\': \'mirage\',\n  \'team\': \'natus-vincere\',\n  \'analytics_map_stats_pick_percentage\': \'39%\',\n  \'analytics_map_stats_ban_percentage\': \'0%\',\n  \'analytics_map_stats_win_percentage\': \'29%\',\n  \'analytics_map_stats_played\': \'7\',\n  \'analytics_map_stats_comment\': \'First pick\',\n  \'match_id\': \'2361342\'},\n    ...\n]\n```\n```match.parse_analytics_center()``` method combines all above methods and returns a tuple of lists.\n\n#### ```HLTVTeam``` provides public methods for parsing [Team stats page]( https://www.hltv.org/stats/teams), with filtering by time using ```time_filter``` parameter, use ```filename``` parameter to save data to a file.\n```sh\nfrom hltv_stats import HLTVTeam\nteam = HLTVTeam("/4608/natus-vincere")\n#You can use match_id to assign team\'s current state(statistic) to specific match\n#this will add match_id field to all json files\nteam.match_id = match.match_id\n```\n```sh\n#time_filter: 3 - last 3 months, 6 - last 6 months, 0 - all time, ...\nteam.parse_matches(time_filter=1) #returns list of played matches in json format\nteam.parse_players(time_filter=1) #returns list of team players\' statistics in json format\nteam.parse_maps(time_filter=1) #returns maps statistics in json format\nteam.parse_events(time_filter=1) #returns events statistics in json format\n\nteam.parse_all_stats(time_filter=1)\n```\n```.parse_all_stats(time_filter=1)``` method combines all above methods and returns a tuple of lists.\n\n#### ```parse_upcoming_matches()``` method parses all upcoming matches from [Match page](https://www.hltv.org/matches).\n```sh\n#Parse upcoming matches and save to json files\n#:param months: list of months to parse, i.e. [1,3] <=> [last month, last 3 months]\n#:param with_teams: bool, if True, parse teams\' statistic as well\n\nfrom hltv_stats import parse_upcoming_matches\nparse_upcoming_matches(months=[1], with_teams=True)\n```\n\n\n```\nResult folder tree contains json files with all match data, and team data if with_teams=True.\n.\n├── configs - contains 2 config files: which holds data about all parsed matches and teams and are used by is_parsed() method to check if match is already parsed and skips it.\n├── output\n│   ├── matches \n│   └── teams\n```\n#### Also check out example.ipynb or contact me.\n\n### 🤝 Contributing\n\nContributions, issues and feature requests are welcome!<br />Feel free to check [issues page](https://github.com/a3agalyan/hltv-stats/issues). \n\n### Show your support\n\nGive a ⭐️ if this project helped you!',
    'author': 'Armen A',
    'author_email': 'agalyan.armen@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/a3agalyan/hltv-stats',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import os
from hltv_stats import Parser, HLTVMatch, HLTVTeam
BASE_URL = "https://www.hltv.org"
from loguru import logger

def get_links_upcoming_matches(include_live=True):
    """Get links to random number of live and upcoming(!!!) matches
    :param include_live: bool, include live matches
    """
    matches_page_url = "https://www.hltv.org/matches"
    soup = Parser._soup_from_url(url=matches_page_url)
    matches = soup.find("div", class_="upcomingMatchesSection") \
        .find_all('a', class_="match a-reset", href=True)
    links = []
    for match in matches:
        team_count = match.parent.find_all('div', class_="matchTeamName text-ellipsis")
        if len(team_count) < 2:
            logger.info(match['href'])
            logger.info("Invalid match, waiting for teams")
        else:
            links.append(match['href'])
    if include_live:
        lives = soup.find_all("div", class_="liveMatch")
        lives = list(map(lambda _: _.a['href'], lives))
        links.extend(lives)
    return links


def parse_upcoming_matches(months, with_teams=False):
    """Parse upcoming matches and save to tree of directories in output/
    :param months: list of months to parse, i.e. [1,3] <=> [last month, last 3 months]
    :param with_teams: bool, if True, parse teams' statistic as well
    """
    dir_path = os.path.join(os.getcwd(), "output/")
    matches_path = dir_path + "matches/"
    teams_path = dir_path + "teams/"
    os.makedirs(os.path.dirname(matches_path), exist_ok=True)
    if with_teams:
        os.makedirs(os.path.dirname(teams_path), exist_ok=True)
    matches_link = get_links_upcoming_matches()
    for ind, match_link in enumerate(matches_link):
        logger.info("parsing :", match_link)
        match = HLTVMatch(BASE_URL + match_link)
        if match.is_parsed():
            logger.info("Match already parsed, skipping")
            continue
        match.parse_analytics_center(filename=f"{matches_path}/{match.match_id}")
        teams = match.teams_link
        if with_teams:
            for m in months:
                team1 = HLTVTeam(teams[0])
                team1.match_id = match.match_id
                team1.parse_all_stats(time_filter=m,
                                      filename=f"{teams_path}/{match.match_id}_{team1.team_name.replace('-', '_')}")
                team2 = HLTVTeam(teams[1])
                team2.match_id = match.match_id
                team2.parse_all_stats(time_filter=m,
                                      filename=f"{teams_path}/{match.match_id}_{team2.team_name.replace('-', '_')}")


if __name__ == '__main__':
    parse_upcoming_matches(months=[3], with_teams=True)